#!/usr/bin/python
# install.py - SNAKE routines for setting up and performing an installation

import os, gzip, tempfile
import snake.uri

# FIXME this needs to support various arches
def grubby(kernel,initrd,args='',title='snake'):
    '''Uses grubby to modify the bootloader config to make the given kernel
    and initrd the default bootloader choice. Uses 'snake' for the title if one
    is not given.'''
    # FIXME check to see if 'snake' already exists and add a suffix if needed
    cmd="""/sbin/grubby --make-default --title='%s' --add-kernel=%s --initrd=%s --args='%s'""" % (title,kernel,initrd,args)
    return os.system(cmd)
    # FIXME need to run ybin here on yaboot (Apple) machines

def __file_write(inf,outf,blocksize=4096):
    '''Write the contents of inf to outf'''
    bytes = 0
    while True:
        data = inf.read(blocksize)
        if data:
            outf.write(data)
            bytes = bytes + len(data)
        else:
            break
    inf.close()
    outf.close()
    return bytes

def add_file_to_initrd(addfile,initrd,outfile=None):
    '''Adds the given file to the given initrd. This encodes the relative
    path of the added file into the initrd, so if you want to add a file foo
    in usr/share on the initrd, it needs to be in usr/share/foo locally.'''
    add_command = 'pax -w -a -f %s %s'
    # Create temporary file
    (fd,tmpinitrd) = tempfile.mkstemp('.cpio')
    os.close(fd)
    # uncompress initrd to tmpinitrd
    b = __file_write(gzip.open(initrd),open(tmpinitrd,'wb'))
    # add file to tmpinitrd
    r = os.system(add_command % (tmpinitrd,addfile))
    if r != 0: # IT'S ALL GONE WRONG!!
        os.unlink(tmpinitrd)
        return False
    # compress tmpinitrd to outfile
    if not outfile: 
        # Nothing specified - overwrite the input file
        outfile = initrd
    b = __file_write(open(tmpinitrd),gzip.open(outfile,'wb'))
    # clean up
    os.unlink(tmpinitrd)
    return True

def fetch_and_prep(uri,**kwargs):
    '''Fetches boot images from the given URI for the given arch,
    then updates the bootloader config to boot into the installer.
    Accepted kwargs:
    args=string: append given args to the boot commandline. 
    arch=string: get images for the given arch (e.g. 'xen') instead of the
                 default tree arch.
    status_callback=func(str): call the given function with various messages
                               used to indicate progress.
    ksdata=string: the given string is kickstart data, to be embedded in the
                   initrd and used during installation.'''

    if 'status_callback' in kwargs:
        status = kwargs['status_callback']
    else:
        def status(msg): pass

    if not snake.uri.check(uri):
        status("%s is not a usable SNAKE URI." % uri)
        return False
    
    tree = snake.uri.uri_to_tree(uri)
    if 'arch' in kwargs:
        arch = kwargs['arch']
    else:
        arch = tree.arch
    images = tree.images[arch]

    try:
        kernel_uri = os.path.join(uri,images['kernel'])
        status("Fetching %s kernel from %s" % (arch,kernel_uri))
        kernel = snake.uri.fetchuri(kernel_uri,'/boot',False)

        initrd_uri = os.path.join(uri,images['initrd'])
        status("Fetching %s initrd from %s" % (arch,initrd_uri))
        initrd = snake.uri.fetchuri(initrd_uri,'/boot',False)
    except IOError, e:
        return False

    # If ksdata is set, but empty.. unset it.
    if 'ksdata' in kwargs and not kwargs['ksdata']:
        del kwargs['ksdata']

    if 'ksdata' in kwargs:
        status("Embedding kickstart data in initrd")
        tmpdir = tempfile.mkdtemp()
        cwd = os.getcwd()
        os.chdir(tmpdir)
        outf = open('ks.cfg','w')
        outf.write(kwargs['ksdata'])
        outf.close()
        # FIXME: handle error here
        add_file_to_initrd('ks.cfg',initrd)

    status("Modifying bootloader config")
    # FIXME: get default args in a smarter way
    if 'args' in kwargs:
        args = kwargs['args']
    else:
        # consider also 'ipv6=auto'
        args = "ip=dhcp noipv6 lang=en keymap=us method=%s" % uri

    if 'ksdata' in kwargs:
        args = args + " ks=file:/ks.cfg"

    snake.install.grubby(kernel,initrd,args,"install %s" % str(tree))

    return True
